@@ -45,9 +45,8 @@ import logging
 import sys
 import os
 from logging.config import fileConfig
-
 from recipes_267662 import wrap_onspace_strict, indent
-from bn import AttributeDict
+from bn import AttributeDict, OrderedDict, MarbleLike
 
 log = logging.getLogger(__name__)
 
@@ -123,7 +122,9 @@ def check_command(name, command):
                 command.arg_spec,
             )
         )
-    if isinstance(command, dict):
+    if isinstance(command, (tuple, list, dict)):
+        if isinstance(command, (tuple, list)):
+            command = OrderedDict(command)
         for name_, command_ in command.items():
             check_command(name_, command_)
     else:
@@ -305,7 +306,7 @@ def assemble_help(
         if template is None:
             if name is None:
                 # This is a main command help
-                template = """%(summary)s
+                template = """%(summary)s%(extra)s
 %(usage)s
 
 %(global_options)s
@@ -316,7 +317,7 @@ def assemble_help(
 
 %(tip)s"""
             else:
-                template = """%(summary)s
+                template = """%(summary)s%(extra)s
 %(usage)s
 
 %(sub_command_options)s
@@ -329,8 +330,14 @@ def assemble_help(
             summary = ''
         else:
             summary = summary
+        extra = commands[name].help.get('extra')
+        if extra is None:
+            extra = ''
+        else:
+            extra = '%s\n'%extra
         help = (template % dict(
             summary = summary,
+            extra = extra,
             usage = usage(commands, name, program),
             global_options = global_options(commands, name, program),
             global_args = global_args(commands, name, program),
@@ -416,7 +423,44 @@ help_option = dict(
     help = 'display this message'
 )
 
-class Cmd(object):
+def on_initial_convert(service, arg_spec, option_spec, raw_args, parsed_options):
+    args = raw_args
+    processed = to_internal(option_spec, parsed_options)
+    opts = processed.opts
+    help_needed = opts.get('help') and True or False
+    if not help_needed:
+        if arg_spec:
+            error = None
+            if isinstance(arg_spec[-1][0], int):
+                min_args = len(arg_spec)-1+arg_spec[-1][0]
+                if len(args) < min_args:
+                    # We might be able to get an error
+                   # import pdb; pdb.set_trace()
+                    if len(arg_spec[-1]) < 3:
+                        error = 'Not enough arguments'
+                    else:
+                        error = arg_spec[-1][2]
+            else:
+                if len(args) > len(arg_spec):
+                    error = 'Unexpected argument %r'%args[len(arg_spec)-1]
+                elif len(args) < len(arg_spec):
+                    if len(arg_spec[len(args)]) < 3:
+                        error = 'Not enough arguments'
+                    else:
+                        error = arg_spec[len(args)][2]
+            if error:
+                raise getopt.GetoptError(error)
+        elif len(args) > 1:
+            raise getopt.GetoptError(
+                'Unexpected arguments: %s'%(
+                    ', '.join([str(arg) for arg in args])[:]
+                )
+            )
+        elif len(args):
+            raise getopt.GetoptError('Unexpected argument %s'%(args[0]))
+    return args, opts, help_needed
+
+class Cmd(MarbleLike):
 
     #
     # Internal code
@@ -424,52 +468,12 @@ class Cmd(object):
 
     default_aliases = {}
 
-    def __init__(self, name=None, aliases=None):
-        self.name = name
-        self.aliases = AttributeDict(self.default_aliases)
-        if aliases:
-            self.aliases.update(aliases)
+    def __init__(self, bag=None, name=None, aliases=None):
+        if bag is None:
+            bag = AttributeDict()
+        aliases = aliases and self.default_aliases.copy().update(aliases) or self.default_aliases.copy()
+        MarbleLike.__init__(self, bag, name, aliases)
 
-    def on_initial_convert(self, service, option_spec, parsed_args, parsed_options):
-        args = parsed_args
-        processed = to_internal(option_spec, parsed_options)
-        opts = processed.opts
-        help_needed = opts.get('help') and True or False
-        if not help_needed:
-            if self.arg_spec:
-                error = None
-                if isinstance(self.arg_spec[-1][0], int):
-                    min_args = len(self.arg_spec)-1+self.arg_spec[-1][0]
-                    if len(args) < min_args:
-                        # We might be able to get an error
-                       # import pdb; pdb.set_trace()
-                        if len(self.arg_spec[-1]) < 3:
-                            error = 'Not enough arguments'
-                        else:
-                            error = self.arg_spec[-1][2]
-                else:
-                    if len(args) > len(self.arg_spec):
-                        error = 'Unexpected argument %r'%args[len(self.arg_spec)-1]
-                    elif len(args) < len(self.arg_spec):
-                        if len(self.arg_spec[len(args)]) < 3:
-                            error = 'Not enough arguments'
-                        else:
-                            error = self.arg_spec[len(args)][2]
-                if error:
-                    raise getopt.GetoptError(error)
-            elif len(args) > 1:
-                raise getopt.GetoptError(
-                    'Unexpected arguments: %s'%(
-                        ', '.join([str(arg) for arg in args])[:]
-                    )
-                )
-            elif len(args):
-                raise getopt.GetoptError('Unexpected argument %s'%(args[0]))
-            self.on_config(service, args, opts, processed)
-        return args, opts, help_needed
-
-    def on_display_help(self, service, help):
-        print help(service)
 
     #
     # Code the user should customize
@@ -485,14 +489,11 @@ class Cmd(object):
         'summary': 'No help summary specified',
     }
 
-    def on_run(self, service, args, opts):
+    def run(self, cmd):
         return 0
 
-    def on_config(self, service, args, opts, processed):
-        return 0
 
 class LoggingCmd(Cmd):
-
     help = {
         'template': None,
         'summary': 'No help summary specified'
@@ -517,44 +518,54 @@ class LoggingCmd(Cmd):
         ),
     }
 
-    def on_config(self, service, args, opts, processed):
-        if opts.get('logging') and (opts.quiet or opts.verbose):
+    def run(self, cmd):
+        if cmd.opts.get('logging') and (cmd.opts.quiet or cmd.opts.verbose):
             raise getopt.GetoptError(
                 'You cannot specify a LOGGING_FILE and also use the '
                 '-q or -v options'
             )
-
-    def on_run(self, service, args, opts):
-        if opts.get('logging'):
-            if not os.path.exists(opts.logging):
-                raise getopt.GetoptError('No such file %r'%opts.logging)
-            fileConfig(opts.logging)
+        if cmd.opts.get('logging'):
+            if not os.path.exists(cmd.opts.logging):
+                raise getopt.GetoptError('No such file %r'%cmd.opts.logging)
+            fileConfig(cmd.opts.logging)
         else:
             format="%(levelname)s: %(message)s"
-            if opts.quiet:
+            if cmd.opts.quiet:
                 logging.basicConfig(level=logging.WARNING, format=format)
-            elif opts.verbose:
+            elif cmd.opts.verbose:
                 logging.basicConfig(level=logging.DEBUG, format=format)
             else:
                 logging.basicConfig(level=logging.INFO, format=format)
 
-
-
-
 def handle_command(commands, cmd_line_parts=None, program=None, service=None, cmd=None):
     # Prepare the cmd object to have the args and opts set
     if cmd is None:
-        cmd = AttributeDict(args=None, opts=None, chain=[])
+        cmd = AttributeDict(
+            args=None, 
+            opts=None, 
+            chain=[], 
+            raw_args=None, 
+            raw_opts=None, 
+            instance=None,
+            service=service,
+        )
     else:
-        cmd.chain.append(AttributeDict(args=cmd.args, opts=cmd.opts))
-	cmd['args'] = None
+        cmd.chain.append(AttributeDict(cmd))
+        # Reset the command
+        cmd['args'] = None
         cmd['opts'] = None
+        cmd['raw_args'] = None
+        cmd['raw_opts'] = None
+        cmd['instance'] = None
+        cmd['service']=service
     if program is None:
         program = sys.argv[0]
         if os.sep in sys.argv[0]:
             program = program.split(os.sep)[-1]
     if cmd_line_parts is None:
         cmd_line_parts = sys.argv[1:]
+    if not isinstance(commands, dict):
+        commands = OrderedDict(commands)
     for name, command in commands.items():
         check_command(name, command)
     if None not in commands:
@@ -567,24 +578,32 @@ def handle_command(commands, cmd_line_parts=None, program=None, service=None, cm
             None, 
             commands[None],
         )
-        k, p, h = commands[None].on_initial_convert(
+        k, p, h = on_initial_convert(
             service, 
+            commands[None].arg_spec, 
             commands[None].option_spec, 
             main_args, 
             main_opts,
         )
         if h:
             help = assemble_help(commands, None, program)
-            commands[None].on_display_help(service, help)
+            print help(service)
             return 0
         # Need to be smarter here, want to call the inner functions in the scope of the outer.
         cmd['args'] = k
         cmd['opts'] = p
-        if hasattr(commands[None], 'run'):
-            result = commands[None].run(service, cmd)
-        else:
+        cmd['raw_args'] = main_args
+        cmd['raw_opts'] = main_opts
+        cmd['instance'] = commands[None]
+        cmd['service'] = service
+        # Have to check this first because all commands inherit a run() method from Cmd.
+        if hasattr(commands[None], 'on_run'):
             # Backwards compatibility
             result = commands[None].on_run(service, k, p)
+        elif hasattr(commands[None], 'run'):
+            result = commands[None].run(cmd)
+        else:
+            raise Exception('Unknown command type %r, no run() method'%commands[sub_command])
         if result:
             return result
         # Now try to run the sub-command
@@ -602,10 +621,9 @@ def handle_command(commands, cmd_line_parts=None, program=None, service=None, cm
             sub_command, 
             commands[sub_command],
         )
-        if not hasattr(commands[sub_command], 'on_initial_convert'):
-            raise Exception('The command for %r has no %r attribute'%(sub_command, 'on_initial_convert'))
-        k, p, h = commands[sub_command].on_initial_convert(
+        k, p, h = on_initial_convert(
             service, 
+            commands[sub_command].arg_spec, 
             commands[sub_command].option_spec, 
             sub_args, 
             sub_opts,
@@ -616,18 +634,23 @@ def handle_command(commands, cmd_line_parts=None, program=None, service=None, cm
                 sub_command, 
                 program, 
             )
-            commands[None].on_display_help(service, help)
+            print help(service)
             return 0
-        cmd.chain.append(AttributeDict(args=cmd.args, opts=cmd.opts))
-	cmd['args'] = None
-        cmd['opts'] = None
+        cmd.chain.append(AttributeDict(cmd))
         cmd['args'] = k
         cmd['opts'] = p
-        if hasattr(commands[sub_command], 'run'):
-            result = commands[sub_command].run(service, cmd)
-        else:
+        cmd['raw_args'] = sub_args
+        cmd['raw_opts'] = sub_opts
+        cmd['instance'] = commands[None]
+        cmd['service'] = service
+        # Have to check this first because all commands inherit a run() method from Cmd.
+        if hasattr(commands[sub_command], 'on_run'):
             # Backwards compatibility
             result = commands[sub_command].on_run(service, k, p)
+        elif hasattr(commands[sub_command], 'run'):
+            result = commands[sub_command].run(cmd)
+        else:
+            raise Exception('Unknown command type %r'%commands[sub_command])
     except getopt.GetoptError, err:
         print 'Error:', str(err)
         if sub_command:
